/*
 * SpanDSP - a series of DSP components for telephony
 *
 * version.h - A tag file, so the exact installed revision can be assertained.
 *
 * Written by Steve Underwood <steveu@coppice.org>
 *
 * Copyright (C) 2007 Steve Underwood
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2.1,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#if !defined(_SPANDSP_VERSION_H_)
#define _SPANDSP_VERSION_H_

/* The date and time of the version are in UTC form. */

#define SPANDSP_RELEASE_DATE            20110122
#define SPANDSP_RELEASE_TIME            075024
#define SPANDSP_RELEASE_DATETIME_STRING "20110122 075024"

#endif
/*- End of file ------------------------------------------------------------*/
